ConfirmationWindow = class(WindowBasic);

local Constants = 
{
	Size = { Width = 510, Height = 170 },
	Padding = { Top = 40, Left = 20, Right = 20, Bottom = 50 },
	TextBox = { Spacing = 20 },
	Buttons =
	{
		Padding = { Bottom = 20 },
		Spacing = 10
	}
}

function ConfirmationWindow:Constructor()
	WindowBasic.Constructor(self);
	
	self:SetSize(Turbine.UI.Display.GetSize());
	self:SetMouseVisible(true);
	
	self.MessageWindow = Turbine.UI.Lotro.Window();
	self.MessageWindow:SetParent(self);
	self.MessageWindow:SetSize(Constants.Size.Width, Constants.Size.Height);
	self.MessageWindow:SetPosition((self:GetWidth() - Constants.Size.Width) / 2,(self:GetHeight() - Constants.Size.Height) / 2);
	self.MessageWindow:SetVisible(true);
	self.MessageWindow.Closed = function(sender, args)
		self:Close();
	end

	self.MessageLabel = Turbine.UI.Label();
	self.MessageLabel:SetParent(self.MessageWindow);
	self.MessageLabel:SetPosition(Constants.Padding.Left, Constants.Padding.Top);
	self.MessageLabel:SetSize(self.MessageWindow:GetWidth() - Constants.Padding.Left - Constants.Padding.Right, self.MessageWindow:GetHeight() - Constants.Padding.Top - Constants.Padding.Bottom);
	self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.MessageLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro16);
	self.MessageLabel:SetForeColor(Globals.Colours.Confirmation);
	self.MessageLabel:SetMultiline(true);
	self.MessageLabel:SetText(text);
	
	self.InputBox = CreateTextBox(self.MessageWindow);
	self.InputBox:SetPosition(Constants.Padding.Left, self.MessageWindow:GetHeight() - Constants.Padding.Bottom - self.InputBox:GetHeight());
	self.InputBox:SetWidth(self.MessageWindow:GetWidth() - Constants.Padding.Left - Constants.Padding.Right);
	self.InputBox:SetVisible(false);
	
	local cancelButton = CreateButton(self.MessageWindow);
	cancelButton:SetPosition((self.MessageWindow:GetWidth() - Constants.Buttons.Spacing) / 2 - Globals.Sizes.Button.Width, self.MessageWindow:GetHeight() - Constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	cancelButton:SetText(Strings["cancel"]);
	
	local okButton = CreateButton(self.MessageWindow);
	okButton:SetPosition((self.MessageWindow:GetWidth() + Constants.Buttons.Spacing) / 2, self.MessageWindow:GetHeight() - Constants.Buttons.Padding.Bottom - Globals.Sizes.Button.Height);
	okButton:SetText(Strings["ok"]);

	cancelButton.Click = function(sender,args)
		self:Close();
	end
	
	okButton.Click = function(sender,args)
		self.ButtonClick(sender, { ["Button"] = "OK", ["Text"] = self.InputBox:GetText():Trim() });
		self:Close();
	end

	self.ButtonClick = function() end;
end

function ConfirmationWindow:ShowWindow(title, text, showTextBox, isModal)
	if showTextBox == nil then
		showTextBox = false;
	end
	
	if isModal == nil then
		isModal = true;
	end

	self.MessageWindow:SetText(title);
	self.MessageLabel:SetText(text);
	self.InputBox:SetText("");
	self.InputBox:SetVisible(showTextBox);

	if showTextBox then
		self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
	else
		self.MessageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	end
	
	self:SetMouseVisible(isModal);
	self:SetVisible(true);
	self.MessageWindow:SetVisible(true);

	self:Activate();
	self.MessageWindow:Activate();
	
	if showTextBox then
		self.InputBox:Focus();
	end
end