DropdownList = class();

local Constants =
{
	Border = 2,
	ScrollBarSize = 10,
	SortIcon =
	{
		Width = 15,
		Height = 15,
		LabelWidth = 20,
		DropdownArrow = "Simbo/Poetical/Icons/DropdownArrow.tga"
	},
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function DropdownList:Constructor(parent)
	self.DisplayItem = Turbine.UI.Control();
	self.DisplayItem:SetParent(parent);
	
	self.DisplayItem.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.SelectionList:SetVisible(not self.SelectionList:IsVisible());
			self.SelectionListBox:Focus();
		end
	end
	
	self.DisplayItemBox = Turbine.UI.Label();
	self.DisplayItemBox:SetParent(self.DisplayItem);
	self.DisplayItemBox:SetMouseVisible(false);
	self.DisplayItemBox:SetPosition(Constants.Border, Constants.Border);
	self.DisplayItemBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	
	self.SelectIconLabel = Turbine.UI.Label();
	self.SelectIconLabel:SetParent(self.DisplayItem);
	self.SelectIconLabel:SetMouseVisible(false);

	self.SelectIcon = Turbine.UI.Control();
	self.SelectIcon:SetParent(self.SelectIconLabel);
	self.SelectIcon:SetMouseVisible(false);
	self.SelectIcon:SetSize(Constants.SortIcon.Width, Constants.SortIcon.Height);
	self.SelectIcon:SetBackground(Constants.SortIcon.DropdownArrow);
	
	self.SelectionList = Turbine.UI.Control();
	self.SelectionList:SetParent(parent);
	self.SelectionList:SetZOrder(parent:GetZOrder() + 100);
	self.SelectionList:SetVisible(false);
	
	self.SelectionListBox = Turbine.UI.ListBox();
	self.SelectionListBox:SetParent(self.SelectionList);
	self.SelectionListBox:SetPosition(Constants.Border, Constants.Border);
	
	self.FireEvent = true;
	
	self.SelectionListBox.SelectedIndexChanged = function(sender, args)
		local selectedLabel = self.SelectionListBox:GetSelectedItem();
		local selectedText = selectedLabel:GetText();
		
		self.DisplayItemBox:SetText(selectedText);

		if self.FireEvent then
			self.SelectedIndexChanged(self, 
			{ 
				["SelectedIndex"] = self.SelectionListBox:GetSelectedIndex(),
				["SelectedValue"] = selectedLabel.Value,
				["SelectedText"] = selectedText;
			});
		end
		
		self.FireEvent = true;
	end
	
	self.SelectionListBox.FocusLost = function(sender, args)
		if not self.LinesScrollBar.HasFocus then
			self.SelectionList:SetVisible(false);
		end
	end
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.SelectionListBox);
	self.LinesScrollBar.HasFocus = false;
	self.SelectionListBox:SetVerticalScrollBar(self.LinesScrollBar);
	self.LinesScrollBar.MouseEnter = function(sender, args)
		self.LinesScrollBar.HasFocus = true;
	end
	self.LinesScrollBar.MouseLeave = function(sender, args)
		self.LinesScrollBar.HasFocus = false;
		self.SelectionListBox:Focus();
	end

	self.SelectedIndexChanged = function() end;
end

function DropdownList:SetPosition(left, top)
	self.DisplayItem:SetPosition(left, top);
	self.SelectionList:SetPosition(self.DisplayItem:GetLeft(), self.DisplayItem:GetTop() + self.DisplayItem:GetHeight() + (Constants.Border * 2));	
end

function DropdownList:SetSize(width, height, dropdownHeight)
	self.DisplayItem:SetSize(width, height);
	self.DisplayItemBox:SetSize(self.DisplayItem:GetWidth() - (Constants.Border * 2) - Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));

	self.SelectIconLabel:SetSize(Constants.SortIcon.LabelWidth, self.DisplayItem:GetHeight() - (Constants.Border * 2));
	self.SelectIconLabel:SetPosition(self.DisplayItemBox:GetLeft() + self.DisplayItemBox:GetWidth(), Constants.Border);
	self.SelectIcon:SetPosition((self.SelectIconLabel:GetWidth() - Constants.SortIcon.Width) / 2, (self.SelectIconLabel:GetHeight() - Constants.SortIcon.Height) / 2);

	self.SelectionList:SetSize(width, dropdownHeight);
	self.SelectionList:SetPosition(self.DisplayItem:GetLeft(), self.DisplayItem:GetTop() + self.DisplayItem:GetHeight() + (Constants.Border * 2));	
	self.SelectionListBox:SetSize(self.DisplayItem:GetWidth() - (Constants.Border * 2), dropdownHeight - (Constants.Border * 2));
	
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize,  self.SelectionListBox:GetHeight());
	self.LinesScrollBar:SetPosition(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize,  0);
end

function DropdownList:SetBorderColor(colour)
	self.DisplayItem:SetBackColor(colour);
	self.SelectionList:SetBackColor(colour);
end

function DropdownList:SetBackColor(colour)
	self.DisplayItemBox:SetBackColor(colour);
	self.SelectIconLabel:SetBackColor(colour);	
	self.SelectionListBox:SetBackColor(colour);	
end

function DropdownList:SetForeColor(colour)
	self.DisplayItemBox:SetForeColor(colour);
end

function DropdownList:SetFont(font)
	self.DisplayItemBox:SetFont(font);
end

function DropdownList:GetSelectedIndex()
	return self.SelectionListBox:GetSelectedIndex();
end

function DropdownList:GetSelectedValue()
	local index = self.SelectionListBox:GetSelectedIndex();
	if index > 0 then
		return self.SelectionListBox:GetItem(index).Value;
	end
	
	return nil;
end

function DropdownList:SetSelectedValue(value, fireEvent)
	local index = 0;
	local checkIndex = 1;
	local itemCount = self.SelectionListBox:GetItemCount();
	while checkIndex <= itemCount and index == 0 do
		if self.SelectionListBox:GetItem(checkIndex).Value == value then
			index = checkIndex;
		else
			checkIndex = checkIndex + 1;
		end
	end
	
	if index > 0 then
		self:SetSelectedIndex(index, fireEvent);
	end
end
	
function DropdownList:SetSelectedIndex(index, fireEvent)
	if index ~= self.SelectionListBox:GetSelectedIndex() then
		if fireEvent == nil then
			fireEvent = true;
		end
	
		self.FireEvent = fireEvent;
		self.SelectionListBox:SetSelectedIndex(index);
	end
end
	
function DropdownList:AddItem(text, value)
	local item = Turbine.UI.Label();
	item:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	item:SetSize(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize, self.DisplayItemBox:GetHeight());
	item:SetFont(self.DisplayItemBox:GetFont());
	item:SetText(text);
	if value == nil then
		item.Value = text;
	else
		item.Value = value;
	end
	self.SelectionListBox:AddItem(item);
	
	item.MouseClick = function(sender, args)
		self.SelectionList:SetVisible(false);
	end
	
	item.MouseEnter = function(sender, args)
		item:SetOutlineColor(Constants.Colours.HoverOutline);
		item:SetFontStyle(Turbine.UI.FontStyle.Outline);
	end
	item.MouseLeave = function(sender, args)
		item:SetOutlineColor(Constants.Colours.NormalOutline);
		item:SetFontStyle(Turbine.UI.FontStyle.None);
	end
end

function DropdownList:ClearItems()
	self.SelectionListBox:ClearItems();
	self.DisplayItemBox:SetText("");
end