PoeticalLine = class(Turbine.UI.Control);

local Constants =
{
	LineSize = 20,
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25)
	}
}

function PoeticalLine:Constructor(parent, id, text, isSelectable, showButton, alias)
	Turbine.UI.Control.Constructor(self);

	self.Parent = parent;
	self.Selected = false;
	self.Id = id;
	self.ShowButton = showButton;

	self:SetParent(parent);
	
	self:SetSelected(false);
	
	self.Label = Turbine.UI.Label();
	self.Label:SetParent(self);
	self.Label:SetMultiline(false);
	self.Label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.Label:SetFont(Globals.Fonts.ListBox);
	self.Label:SetText(text);
	
	if self.ShowButton then
		self.Shortcut = Turbine.UI.Lotro.Shortcut();
		self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
		self.Shortcut:SetData(alias);
		
		self.Quickslot = Turbine.UI.Lotro.Quickslot();
		self.Quickslot:SetParent(self);
		self.Quickslot:SetSize(Constants.LineSize - 2, Constants.LineSize - 2);
		self.Quickslot:GetAllowDrop(false);
		self.Quickslot:SetShortcut(self.Shortcut);
		
		-- Fix for dragging short-cut
		self.Quickslot.DragDrop = function(sender, args)
			self.Quickslot:SetShortcut(self.Shortcut); -- restore shortcut
		end
		
		self.Icon = self:CreateMainIcon()
				
		self.Quickslot.MouseEnter = function(sender, args)
			self.Icon:SetBackground(Globals.Icons.LineShortcut_Hover);
			self.Parent.QuickslotClicked = true;
		end
		self.Quickslot.MouseLeave = function(sender, args)
			self.Icon:SetBackground(Globals.Icons.LineShortcut);
			self.Parent.QuickslotClicked = false;
		end
	end
	
	self:SetSelectable(isSelectable);
	
	self.MouseEnter = function(sender, args)
		if self.IsSelectable then
			self.Label:SetOutlineColor(Constants.Colours.HoverOutline);
			self.Label:SetFontStyle(Turbine.UI.FontStyle.Outline);
		end
	end
	self.MouseLeave = function(sender, args)
		if self.IsSelectable then
			self.Label:SetOutlineColor(Constants.Colours.NormalOutline);
			self.Label:SetFontStyle(Turbine.UI.FontStyle.None);
		end
	end
		
	self:LayoutControls();
end

function PoeticalLine:LayoutControls()
	self:SetSize(self.Parent:GetWidth(), Constants.LineSize);

	self.Label:SetPosition(0, 0);
	self.Label:SetSize(self:GetWidth() - Constants.LineSize, Constants.LineSize);

	if self.ShowButton then
		if self.IsSelectable then
			self.Quickslot:SetPosition(self.Label:GetWidth(), 0);
			self.Icon:SetPosition(self.Label:GetWidth(), 0);

			self.Quickslot:SetVisible(true);
			self.Icon:SetVisible(true);
		else
			self.Quickslot:SetVisible(false);
			self.Icon:SetVisible(false);
		end
	end
end

function PoeticalLine:Destructor()
	if self.ShowButton then
		self.Quickslot = nil;
	end
end

function PoeticalLine:CreateMainIcon()
	local icon = Turbine.UI.Control();
	icon:SetParent(self);
	icon:SetSize(Constants.LineSize, Constants.LineSize);
	icon:SetZOrder(110);
	icon:SetMouseVisible(false);
	icon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	icon:SetBackground(Globals.Icons.LineShortcut);
	return icon;
end

function PoeticalLine:SetTextOnly(text, alias)
	self:SetTextAndId(text, alias, nil);
end

function PoeticalLine:SetColour(color)
	self.Label:SetForeColor(color);
end

function PoeticalLine:SetStandardColour()
	if self.IsSelectable then
		self.Label:SetForeColor(Globals.Colours.ListBox);
	else
		self.Label:SetForeColor(Globals.Colours.SecondaryLabel);
	end
end

function PoeticalLine:GetAlias()
	if self.IsSelectable and self.ShowButton then
		return self.Shortcut:GetData();
	end
	
	return self.Label:GetText();
end

function PoeticalLine:SetTextAndId(text, alias, id)
	self.Label:SetText(text);
	if id ~= nil then
		self.Id = id;
	end
	if self.IsSelectable and self.ShowButton then
		self.Shortcut:SetData(alias);
		self.Quickslot:SetShortcut(self.Shortcut);
	end
end

function PoeticalLine:SetSelected(selected)
	self.Selected = selected;
	if self.Selected then
		self:SetBackColor(Constants.Colours.SelectedItem);
	else
		self:SetBackColor(self.Parent:GetBackColor());
	end
end

function PoeticalLine:SetSelectable(selectable)
	self.IsSelectable = selectable;
	self:SetStandardColour();
end
