PoeticalListBox = class(Turbine.UI.Control);

local Constants =
{
	Border = 1,
	ScrollBarSize = 10
}

function PoeticalListBox:Constructor(parent, showButton, initialHeight, quickslot)
	Turbine.UI.Control.Constructor(self);

	self.Parent = parent;
	self.ShowButton = showButton;
	self.Quickslot = quickslot;
	self.LineCount = 0;
	self.SelectedIndex = -1;
	self.Lines = {};
	
	self.SelectedIndexChanged = nil;
	
	self:SetParent(self.Parent);
	self:SetBackColor(Globals.BorderColours.ListBox);
	if initialHeight ~= nil then
		self:SetHeight(initialHeight);
	end
	
	self.LinesBox = Turbine.UI.ListBox();
	self.LinesBox:SetParent(self);
	self.LinesBox:SetBackColor(Globals.BackgroundColours.ListBox);
	self.LinesBox.ResetIndex = false;
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.Parent);
	self.LinesBox:SetVerticalScrollBar(self.LinesScrollBar);

	self.LinesBox.SelectedIndexChanged = function(sender, args)
		local index = self.LinesBox:GetSelectedIndex();
		if index > 0 then
			if not self.LinesBox.ResetIndex then
				if self.LinesBox.QuickslotClicked == nil or not self.LinesBox.QuickslotClicked then
					self:SelectLine(index);
				end
			end
			self.LinesBox.ResetIndex = false;
			self.LinesBox:SetSelectedIndex(0);
		else
			self.LinesBox.ResetIndex = false;
		end
	end
	
	self.SelectedIndexChanged = function() end;
end

function PoeticalListBox:AdjustHeight(height)
	self:AdjustSizeAndPosition(self:GetLeft(), self:GetTop(), self:GetWidth() + Constants.ScrollBarSize, height)
end

function PoeticalListBox:AdjustTopAndHeight(top, height)
	self:AdjustSizeAndPosition(self:GetLeft(), top, self:GetWidth() + Constants.ScrollBarSize, height)
end

function PoeticalListBox:AdjustSizeAndPosition(left, top, width, height)
	self:SetPosition(left, top);
	self:SetSize(width - Constants.ScrollBarSize, height);

	self.LinesBox:SetPosition(Constants.Border, Constants.Border);
	self.LinesBox:SetSize(self:GetWidth() - Constants.Border * 2, self:GetHeight() - Constants.Border * 2);
	
	self.LinesScrollBar:SetPosition(self:GetLeft() + self:GetWidth(), self:GetTop());
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize, self:GetHeight());
	
	for i = 1, self.LinesBox:GetItemCount() do
		self.LinesBox:GetItem(i):LayoutControls();
	end
end

function PoeticalListBox:Clear()
	self.LinesBox:ClearItems();
	self.LineCount = 0;
	self.SelectedIndex = -1;
	self:EnableQuickSlot(false);
	
	local index = table.getn(self.Lines);
	while index > 0 do
		self.Lines[index]:SetTextAndId("", "", nil);
		self.Lines[index]:SetSelected(false);
		index = index - 1;
	end
end

function PoeticalListBox:AddLine(text, id, isSelectable, alias, colour)
	local line = nil;
	
	if alias == nil then
		alias = text;
	end
	
	if isSelectable == nil then
		isSelectable = true;
	end
	
	if self.LineCount < table.getn(self.Lines) then
		line = self.Lines[self.LineCount + 1];
		line:SetSelectable(isSelectable);
		line:SetTextAndId(text, alias, id);
		line:SetSelected(false);
	else
		line = PoeticalLine(self.LinesBox, id, text, isSelectable, self.ShowButton, alias);
		table.insert(self.Lines, self.LineCount + 1, line);
	end
	
	if colour ~= nil then
		line:SetColour(colour);
	else
		line:SetStandardColour(colour);
	end
	
	self.LinesBox:AddItem(line);
	line:LayoutControls();
	
	self.LineCount = self.LineCount + 1;
end

function PoeticalListBox:UpdateLine(index, text, alias)
	if alias == nil then
		alias = text;
	end
	
	self.LinesBox:GetItem(index):SetTextOnly(text, alias);
end

function PoeticalListBox:SelectLineById(id, scrollToTop)
	local index = 1;
	local checkLine;
	local found = false;
	
	while index <= self.LineCount and not found do
		checkLine = self.LinesBox:GetItem(index);
		if checkLine.Id == id then
			found = true;
		else
			index = index + 1;
		end
	end
	
	if found then
		self:SelectLine(index, scrollToTop);
	end
end

function PoeticalListBox:SelectLine(index, scrollToTop)
	local selectedLine = nil;
	local foundLine = false;

	if index > self.LineCount then
		index = 0;
	end

	selectedLine = self:HighlightLine(index);
	
	local selectedId;
	if selectedLine ~= nil then
		selectedId = selectedLine.Id;
		
		if scrollToTop ~= nil and scrollToTop then
			if self.SelectedIndex > 0 then
				self.ResetIndex = true;
				self.LinesBox:SetSelectedIndex(self.SelectedIndex);
			end
			--self.LinesBox:SetSelectedIndex(0);
		end
	else
		self:EnableQuickSlot(false);
	end
	
	local args = 
	{
		SelectedIndex = self.SelectedIndex,
		SelectedId = selectedId  
	}; 
	
	self.SelectedIndexChanged(self, args);
end

function PoeticalListBox:SelectNextLine(fromIndex)
	local index = fromIndex + 1;
	local checkLine;
	local found = false;
	
	while index <= self.LineCount and not found do
		checkLine = self.LinesBox:GetItem(index);
		if checkLine.IsSelectable then
			found = true;
		else
			index = index + 1;
		end
	end
	
	self:SelectLine(index);
	self:ScrollIntoView();
end

function PoeticalListBox:HighlightLine(index)
	local selectedLine = nil;
	
	if index >= 1 then
		selectedLine = self.LinesBox:GetItem(index);
	end
	
	if selectedLine == nil or selectedLine.IsSelectable then
		if self.SelectedIndex >= 1 then
			self.LinesBox:GetItem(self.SelectedIndex):SetSelected(false);
		end

		self.SelectedIndex = index;
		
		if selectedLine ~= nil then
			selectedLine:SetSelected(true);
			if self.Quickslot ~= nil then
				--local shortcut = self.Quickslot:GetShortcut();
				--shortcut:SetData(selectedLine:GetAlias());
				self.Quickslot:SetShortcut(selectedLine:GetAlias());
				self:EnableQuickSlot(true);
			end
		end
	end
	
	return selectedLine;
end

function PoeticalListBox:GetSelectedLineIndex()
	return self.SelectedIndex;
end

function PoeticalListBox:GetSelectedLineId()
	if self.SelectedIndex >= 1 then
		return self.LinesBox:GetItem(self.SelectedIndex).Id;
	else
		return nil;
	end
end

function PoeticalListBox:GetIndexForId(id)
	local selectedIndex = nil;
	local index = 1;
	
	while selectedIndex == nil and index <= self.LineCount do
		if self.LinesBox:GetItem(index).Id == id then
			selectedIndex = index;
		end
		index = index + 1;
	end
	
	return selectedIndex;
end

function PoeticalListBox:EnableQuickSlot(enabled)
	if self.Quickslot ~= nil then
		self.Quickslot:SetEnabled(enabled);
	end
end

function PoeticalListBox:ScrollIntoView()
	if self.SelectedIndex > 0 then
		local item = self.LinesBox:GetItemAt(0, 16);
		if item ~= nil then
			local topIndex = self.LinesBox:IndexOfItem(item);
			local newIndex = 0;
			if topIndex > self.SelectedIndex then
				newIndex = self.SelectedIndex;
			else
				item = self.LinesBox:GetItemAt(0, self.LinesBox:GetHeight() - 16);
				local bottomIndex = self.LinesBox:IndexOfItem(item);
				if bottomIndex < self.SelectedIndex then
					newIndex = self.SelectedIndex - (bottomIndex - topIndex);
				end
			end
			
			if newIndex > 0 then
				if newIndex > self.LinesBox:GetItemCount() then
					newIndex = self.LinesBox:GetItemCount();
				end
				self.LinesBox.ResetIndex = true;
				self.LinesBox:SetSelectedIndex(newIndex);
			end
		end
	end
end