ReadButton = class();

local Constants =
{
	Size = 
	{ 
		Width = 32, 
		Height = 32 
	}
}

function ReadButton:Constructor(parent)
	self.UpdateState = 0;
	self.DelayUntil = 0;
	self.UpdateCount = 0;
	self.UpdateControl = Turbine.UI.Control();

	self.Shortcut = Turbine.UI.Lotro.Shortcut();
	self.Shortcut:SetType(Turbine.UI.Lotro.ShortcutType.Alias);
	self.Shortcut:SetData("/");
	
	self.Quickslot = Turbine.UI.Lotro.Quickslot();
	self.Quickslot:SetParent(parent);
	self.Quickslot:SetSize(Constants.Size.Width - 2, Constants.Size.Height - 2);
	self.Quickslot:GetAllowDrop(false);
	self.Quickslot:SetShortcut(self.Shortcut);
	self.Quickslot:SetVisible(false);
	
	-- Fix for dragging short-cut
	self.Quickslot.DragDrop = function(sender, args)
		self.Quickslot:SetShortcut(self.Shortcut); -- restore shortcut
	end
	
	self.QuickslotIcon = Turbine.UI.Control();
	self.QuickslotIcon:SetParent(parent);
	self.QuickslotIcon:SetSize(Constants.Size.Width, Constants.Size.Height);
	self.QuickslotIcon:SetZOrder(self.Quickslot:GetZOrder() + 1);
	self.QuickslotIcon:SetMouseVisible(false);
	self.QuickslotIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
	self.QuickslotIcon.EnabledChanged = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
		else
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Inactive);
		end
	end
	self.QuickslotIcon:SetEnabled(false);
	
	self.Quickslot.MouseEnter = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut_Hover);
		end
	end
	self.Quickslot.MouseLeave = function(sender, args)
		if self.QuickslotIcon:IsEnabled() then
			self.QuickslotIcon:SetBackground(Globals.Icons.MainShortcut);
		end
	end
	self.Quickslot.MouseClick = function(sender, args)
		self.UpdateState = 1;
		self.UpdateControl:SetWantsUpdates(true);
	end
	
	self.UpdateControl.Update = function(sender, args)
		local delay = tonumber(Settings.Delay) or 0;
		local localTime = nil;
		
		if self.UpdateState == 1 then
			self.ButtonPressed(self, {});
			self.UpdateCount = 0;
			if self.QuickslotIcon:IsEnabled() and delay > 0 then
				self.UpdateControl:SetWantsUpdates(true);
				self.UpdateState = 2;
				
				localTime = Turbine.Engine.GetLocalTime();
				self.DelayUntil = localTime + delay;
				self:SetEnabled(false, false);
			else
				self:CancelUpdate();
			end
		end
		
		if self.UpdateControl:GetWantsUpdates() then
			if localTime == nil then
				localTime = Turbine.Engine.GetLocalTime();
			end
			
			self.UpdateCount = self.UpdateCount + 1;
			
			if localTime >= self.DelayUntil and self.UpdateCount > 50 then
				self:SetEnabled(true);
				self.UpdateCount = 0;
			end
		end
	end
	
	self.ButtonPressed = function(sender, args) end;
end

function ReadButton:SetPosition(left, top)
	self.QuickslotIcon:SetPosition(left, top);
	self.Quickslot:SetPosition(self.QuickslotIcon:GetLeft() + 1, self.QuickslotIcon:GetTop() + 1);
end

function ReadButton:GetWidth()
	return Constants.Size.Width;
end

function ReadButton:GetHeight()
	return Constants.Size.Height;
end

--function ReadButton:GetShortcut()
--	return self.Quickslot:GetShortcut();
--end

function ReadButton:SetShortcut(shortcutAlias)
	--self.Shortcut = shortcut;
	self.Shortcut:SetData(shortcutAlias);
	self.Quickslot:SetShortcut(self.Shortcut);
end

function ReadButton:SetEnabled(enabled, cancelUpdate)
	self.Quickslot:SetVisible(enabled);
	self.QuickslotIcon:SetEnabled(enabled);
	
	if cancelUpdate == nil or cancelUpdate then
		self:CancelUpdate();
	end
end

function ReadButton:CancelUpdate()
	self.UpdateControl:SetWantsUpdates(false);
	self.UpdateState = 0;
	self.DelayUntil = 0;
end